
/**
 * fonction chargée de créer la base de donnée si elle n'existe pas
 * ou si elle nécessite une mise à jour.
 * Appellée sur le onupgradeneeded à l'ouverture de la base
 */
function createDatabase(event) {
    var db = event.target.transaction.db;
    
    // création d'un object store (similaire à une table en SQL)
    // la clé d'un enregistrement se trouve dans la propriété id
    // de l'enregistrement, et initialisé automatiquement à l'insertion
    // avec une valeur d'un compteur auto incrémenté
    var voitureStore = db.createObjectStore("voitures",
                                            { keyPath: "id",
                                              autoIncrement: true });

    // création d'un index sur cet object store, sur la propriété modele
    // des objets enregistrés
    //voitureStore.createIndex("modeles", "modele", { unique: false });
}

/**
 * le callback quand il y a une erreur à l'ouverture
 */
function errorOpen(event) {
    window.alert("Erreur ouverture !");
}

    

/**
 * fonction appelée lors de la soumission du formulaire.
 * ajoute un enregistrement dans la base
 */
function saveRecord(form) {
    // création d'un objet contenant les données
    // il sert d'"enregistrement" dans la base
    var voiture = {
        model:  form.elements['model'].value,
        maker:  form.elements['maker'].value,
        year:   form.elements['year'].value
    }

    // on ouvre la base, et on déclare les listeners
    var request = indexedDB.open("vehicules", 1);
    request.onerror = errorOpen;
    request.onupgradeneeded = createDatabase;

    request.onsuccess = function(event) {
        // ici la base a été ouverte avec succés, il faut ajouter l'enregistrement

        // on récupère l'objet database
        var db = event.target.result; 

        // on ouvre une transaction qui permettra d'effectuer
        // les opérations sur la base
        var transaction = db.transaction(["voitures"], "readwrite");
        transaction.oncomplete = function(event) {
            displayList(db);
            window.alert("Transaction terminée, sauvegarde effectuée");
        };

        transaction.onerror = function(event) {
           window.alert('erreur de transaction ');
        };

        // on récupère l'object store dans lequel on veut stocker l'objet
        var voitureStore = transaction.objectStore("voitures");

        // on créé l'ordre d'ajouter un enregistrement
        // sera effectivement executé lors de la fermeture de la transaction
        var req = voitureStore.add(voiture);
        req.onsuccess = function(event) {
            
        }
        req.onerror = function(event) {
            window.alert('erreur ajout');
        }
    }
}


/**
 * fonction qui efface tout le contenu de la base, quand on
 * clique sur le bouton "effacer" de la page
 */
function deleteAllRecords() {
    // on ouvre la base, et on déclare les listeners
    var request = indexedDB.open("vehicules", 1);
    request.onerror = errorOpen;
    request.onupgradeneeded = createDatabase;

    request.onsuccess = function(event) {
        var db = event.target.result;

        // on ouvre une transaction qui permettra d'effectuer la suppression
        var transaction = db.transaction(["voitures"], "readwrite");
        transaction.oncomplete = function(event) {displayList(db);};
        transaction.onerror = function(event) {
           window.alert('erreur de transaction suppression');
        };

        var voitureStore = transaction.objectStore("voitures");
        var req = voitureStore.clear();
        req.onsuccess = function(event) {
        }
        req.onerror = function(event) {
            window.alert('erreur suppression');
        }
    }
}

// on ajoute un listener sur le load de la fenêtre, pour remplir
// la liste des véhicules, si elle existe
window.addEventListener('load', function(event) {
    // on ouvre la base, et on déclare les listeners
    var request = indexedDB.open("vehicules", 1);
    request.onerror = errorOpen;
    request.onupgradeneeded = createDatabase;

    request.onsuccess = function(event) {
        displayList(event.target.result);
    }
}, false);




function displayList(db) {

    // on ouvre une transaction qui permettra d'effectuer
    // la lecture. uniquement de la lecture -> "readonly"
    var transaction = db.transaction(["voitures"], "readonly");
    transaction.oncomplete = function(event) {};
    transaction.onerror = function(event) {
       window.alert('erreur de transaction lecture ');
    };

    // on en profite pour effacer le formulaire
    var form = document.getElementById('voitureform');
    form.elements['model'].value = ''
    form.elements['maker'].value = ''
    form.elements['year'].value = ''
    
    // récupération de la table html
    var list = document.getElementById("listvehicule");
    
    // on y efface tout
    list.innerHTML = '';

    // on récupère l'object store que l'on veut lire
    var voitureStore = transaction.objectStore("voitures");
    voitureStore.openCursor().onsuccess = function (event) {

    var cursor = event.target.result;
        if (cursor) {
            var vehicule = cursor.value; // un enregistrement

            // création de la ligne html dans le tableau
            var tr = document.createElement('tr');

            var tdmodel = document.createElement('td');
            tdmodel.textContent = vehicule.model;
            tr.appendChild(tdmodel);

            var tdconstructor = document.createElement('td');
            tdconstructor.textContent = vehicule.maker;
            tr.appendChild(tdconstructor);

            var tdannee = document.createElement('td');
            tdannee.textContent = vehicule.year;
            tr.appendChild(tdannee);

            list.appendChild(tr);

            // on avance le curseur -> la callback onsuccess
            // sera appelée à nouveau
            cursor.continue();
        }
    }
}

